package main

import (
	"flag"
	"log"
	"math/rand"
	"time"
)

type RepoUpdate struct {
	repo Repo
	Tag  string
}

func notifierThread(c chan RepoUpdate, n Notifier) {
	update := <-c
	log.Printf("New tag stored for %s: %s", update.repo.IdTuple(), update.Tag)
	if n.Notify(update.repo, update.Tag) {
		log.Printf("Notified for %s: %s", update.repo.IdTuple(), update.Tag)
	}
	notifierThread(c, n)
}

func repoThread(c chan RepoUpdate, repo Repo, avgInterval int, db Db) {
	log.Println("Checking", repo.IdTuple())
	tag, err := repo.GetLatestTag()
	isNewVersion, err_2 := db.CheckAndStore(repo.IdTuple(), tag.Name)

	if err != nil {
		log.Printf("Failed to fetch latest tag for %s: %v", repo.IdTuple(), err)
	} else if err_2 != nil {
		log.Printf("Failed to store latest tag for %s: %v", repo.IdTuple(), err_2)
	} else if isNewVersion {
		c <- RepoUpdate{repo, tag.Name}
	}

	sleepTime := time.Duration(rand.Intn(avgInterval)) * time.Second
	time.Sleep(sleepTime)

	repoThread(c, repo, avgInterval, db)
}

func main() {

	conf := flag.String("config", "./beretta.json", "The path to the config file")
	flag.Parse()

	log.Printf("Using config path: %s", *conf)

	// Load configuration
	config, err := loadConfig(*conf)
	if err != nil {
		log.Fatalf("Failed to load configuration: %v", err)
	}

	// Create database
	db, err := NewSQLiteDb("tags.db")
	if err != nil {
		log.Fatalf("Failed to create database: %v", err)
	}

	notifier := NtfyNotifier{Topic: config.NtfyTopic}

	c := make(chan RepoUpdate)

	// Spawn a goroutine for each repository
	for _, repo := range config.Repos {
		go repoThread(c, repo, config.Interval, &db)
	}

	notifierThread(c, notifier)
}
